% CSE4421 Matlab Tutorial

% Matlab is a numerical computing environment that was
% originally designed as an interface for using linear algebra
% libraries without having to use Fortran. It has since grown into
% something much more powerful (you can plot data, implement
% algorithms in the Matlab language, create user interfaces,
% perform statistical analysis, control hardware, and the list
% goes on and on).

% Matlab is weakly dynamically typed; you don't have to specify
% a type when you create a variable, and you can assign anything
% to a variable.

% To create a scalar:
x = 1

% To create a row vector:
x = [1 2 3]

% To create a column vector:
x = [1
    2
    3]

% or (semi-colon denotes a new row):
x = [1; 2; 3]

% or (single quote is matrix transpose):
x = [1 2 3]'

% or create a row-vector and use the colon notation:
x = [1 2 3]
x = x(:)

% To create a matrix:
A = [1 2 3
    4 5 6
    7 8 9]

% or (semi-colon denotes a new row):
A = [1 2 3;
    4 5 6;
    7 8 9];

% You can create a matrix of all-zeros:
help zeros
A = zeros(5, 3)

% or ones:
help ones
A = ones(3, 7)

% or an identity matrix:
help eye
A = eye(4)

% All of the natural arithmetic operations are supported:
help arith
A = ones(3, 3)
B = [1 2 3;
    4 5 6;
    7 8 9]
% Matrix addition, subtraction, multiplication:
C = A + B
C = A - B
C = A * B

% You can find out about the relational operators:
help relop

% Indexing is one based (unlike C and its descendents):
B(1, 1)   % row 1, col 1
B(1, 2)   % row 1, col 2
B(1, 3)   % row 1, col 3

% You can retrieve a sub-matrix using indexing and the colon:
C = B(1:2, 1:2)      % upper-left 2x2 sub-matrix of B
C = B(2:3, 2:3)      % lower-right 2x2 sub-matrix of B
C = B(2:end, 1:end)  % bottom 2 rows of B



