
% Functions
% Most functions in Matlab will operate on matrices. Some operate
% on individual elements, some operate on columns of the matrix,
% and some operate on the entire matrix.

% Scalar functions operate element by element; not a complete list:
% sin, cos, tan, acos, asin, atan, atan2, exp, log, abs, sqrt, sign,
% round, floor, ceil, rem

% Example: plotting a circle
angle = 0:10:360      % 0 to 360 degrees in increments of 10
x = cos(angle * pi/180);
y = sin(angle * pi/180);
plot(x, y)
axis equal


% Vector functions usually operate on columns (but some operate
% on rows); not a complete list:
% max, min, sort, sum, prod, median, mean, std, any, all

% Example:
% put x and y into columns of a matrix P
P = [x' y']
hi = max(P)
lo = min(P)
avg = mean(P)

% sort can return both the sorted result and the matrix of indexes
[Q, I] = sort(P)

% Q contains the x and y components of P sorted
% I contains the index into P where each element of Q came from;
% the first row of I is:
I(1, :)

% I(1, 1) is the index in the first column of P where Q(1, 1)
% came from:
Q(1, 1) == P(I(1,1), 1)

% I(1, 2) is the index in the second column of P where Q(1, 2)
% came from:
Q(1, 2) == P(I(1,2), 2)



% Matrix functions operate on matrices; not a complete list:
% inv, transpose, det, size, rank, eig

angle = 30*pi/180;
Rz = [cos(angle) -sin(angle) 0;
      sin(angle)  cos(angle) 0;
      0           0          1]
inv(Rz)
transpose(Rz)
det(Rz)
size(Rz)
rank(Rz)
[V, D] = eig(Rz)

  